use crossbeam::channel::{Receiver, Sender};
use log::{Level, Metadata, Record};
use ratatui::{text::Line, widgets::Wrap};

use crate::ui::Component;

pub struct SimpleLogger {
    sender: Sender<String>,
    pub receiver: crossbeam::channel::Receiver<String>,
}

impl SimpleLogger {
    pub fn new() -> Self {
        let (sender, receiver) = crossbeam::channel::unbounded();
        SimpleLogger { sender, receiver }
    }

    pub fn init(&self) -> Receiver<String> {
        let logger = SimpleLogger::new();
        let receiver = logger.receiver.clone();
        log::set_boxed_logger(Box::new(logger)).unwrap();
        log::set_max_level(log::LevelFilter::Trace);
        return receiver;
    }
}

impl log::Log for SimpleLogger {
    fn enabled(&self, metadata: &Metadata) -> bool {
        metadata.level() <= Level::Trace
    }

    fn log(&self, record: &Record) {
        if self.enabled(record.metadata()) {
            self.sender
                .send(format!("{} - {}", record.level(), record.args()))
                .unwrap();
        }
    }

    fn flush(&self) {}
}

pub struct SimpleLoggerConsumer {
    receiver: crossbeam::channel::Receiver<String>,
    log: Vec<String>,
}

impl SimpleLoggerConsumer {
    pub fn new(receiver: crossbeam::channel::Receiver<String>) -> Self {
        SimpleLoggerConsumer {
            receiver,
            log: vec![],
        }
    }
}

impl Component for SimpleLoggerConsumer {
    fn update(&mut self, event: crate::ui::AppEventClient) {
        //   todo!()
        while self.receiver.len() > 0 {
            match self.receiver.try_recv() {
                Ok(msg) => {
                    while self.log.len() >= 20 {
                        self.log.remove(0);
                    }
                    self.log.push(msg);
                }
                Err(_) => break,
            }
        }
    }

    fn render(&mut self, frame: &mut ratatui::Frame, rect: ratatui::prelude::Rect) {
        let mut lines: Vec<Line> = self.log.iter().map(|l| Line::from(l.as_str())).collect();
        lines.reverse();
        if lines.len() > 20 {
            lines.truncate(20);
        }
        let paragraph = ratatui::widgets::Paragraph::new(lines)
            .block(ratatui::widgets::Block::default())
            .wrap(Wrap::default());
        frame.render_widget(paragraph, rect);
    }
}
